package sensorGlove;

import gnu.io.*; // RXTX

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.TooManyListenersException;

public class Glove extends Thread {

	private final PropertyChangeSupport pcs = new PropertyChangeSupport(this);
	
	public void addPropertyChangeListener(PropertyChangeListener listener) {
        this.pcs.addPropertyChangeListener(listener);
    }

    public void removePropertyChangeListener(PropertyChangeListener listener) {
        this.pcs.removePropertyChangeListener(listener);
    }

	CommPortIdentifier serialPortId;
	Enumeration<?> enumComm;
	SerialPort serialPort;
	InputStream inputStream;
	OutputStream outputStream;
	Boolean serialPortOpen = false;

	int baudrate = 115200;
	int dataBits = SerialPort.DATABITS_8;
	int stopBits = SerialPort.STOPBITS_1;
	int parity = SerialPort.PARITY_NONE;
	int flowControl = SerialPort.FLOWCONTROL_NONE;
	
	String portName = "";
	
	public Glove(String portName){
		this.portName = portName;
	}
	
	public void open(){
		super.start();
	}
	
	@Override
    public void run(){
		openSerialPort(portName);
    }
    
	protected boolean openSerialPort(String portName){
		Boolean foundPort = false;
		if (serialPortOpen != false) {
			System.out.println("Port already open!");
			return false;
		}
		System.out.println("Opening serialport");
		enumComm = CommPortIdentifier.getPortIdentifiers();
		while(enumComm.hasMoreElements()) {
			serialPortId = (CommPortIdentifier) enumComm.nextElement();
			if (portName.contentEquals(serialPortId.getName())) {
				foundPort = true;
				break;
			}
		}
		
		if (foundPort != true) {
			System.out.println("Could not find port: " + portName);
			return false;
		}
		
		try {
			serialPort = (SerialPort) serialPortId.open("Öffnen und Senden", 500);
		} catch (PortInUseException e) {
			System.out.println("Port already in use.");
		}

		try {
			outputStream = serialPort.getOutputStream();
		} catch (IOException e) {
			System.out.println("Can not access output stream.");
		}

		try {
			inputStream = serialPort.getInputStream();
		} catch (IOException e) {
			System.out.println("Can not access input stream.");
		}
		try {
			serialPort.addEventListener(new serialPortEventListener());
		} catch (TooManyListenersException e) {
			System.out.println("TooManyListenersException");
		}
		serialPort.notifyOnDataAvailable(true);
		try {
			serialPort.setSerialPortParams(baudrate, dataBits, stopBits, parity);
			serialPort.setFlowControlMode(flowControl);
		} catch(UnsupportedCommOperationException e) {
			System.out.println("Could not set port parameters");
		}
		
		serialPortOpen = true;
		return true;
	}

	public void close(){
		if ( serialPortOpen == true) {
			System.out.println("Closing serialport");
			setMotorValues(new int[]{0,0,0,0,0});
			serialPort.close();
			serialPortOpen = false;
		} else {
			System.out.println("Serialport already closed");
		}
	}
	
	protected void serialPortDataReady() {
		try {
			byte[] data = new byte[150];
			int num = 0;
			while(inputStream.available() > 0) {
				num = inputStream.read(data, 0, data.length);
			}
			
			int[] flexValues = new int[5];
			int idx = 0;
			boolean found = false;
			for( idx = 1; idx <= (num-17); idx ++){
			
				if( (data[idx] == 114)
						&& (data[idx+3] == 44)
						&& (data[idx+6] == 44)
						&& (data[idx+9] == 44)
						&& (data[idx+12] == 44)
						&& (data[idx+15] == 44)
						&& (data[idx+16] == 120)
						){
					flexValues[0] = ((0xFF & data[idx+1]) << 8) | (0xFF & data[idx+2]);
					flexValues[1] = ((0xFF & data[idx+4]) << 8) | (0xFF & data[idx+5]);
					flexValues[2] = ((0xFF & data[idx+7]) << 8) | (0xFF & data[idx+8]);
					flexValues[3] = ((0xFF & data[idx+10]) << 8) | (0xFF & data[idx+11]);
					flexValues[4] = ((0xFF & data[idx+13]) << 8) | (0xFF & data[idx+14]);
					
					found = true;
					break;
					//this.pcs.firePropertyChange("gloveFlexValues", 0, flexValues);
				}
				
			}
			
			if(found){
				this.pcs.firePropertyChange("gloveFlexValues", 0, flexValues);
			}
			
			
			
			
		} catch (IOException e) {
			System.out.println("Could not read data from serialport");
		}
	}
	
	public void setMotorValues(int values[]){
		if (serialPortOpen != true)
			return;
		try {
			outputStream.write(("w,"+values[0]+","+values[1]+","+values[2]+","+values[3]+","+values[4]+"x").getBytes());
		} catch (IOException e) {
			System.out.println("Could not send data to serialport");
		}
	}
	

	class serialPortEventListener implements SerialPortEventListener {
		public void serialEvent(SerialPortEvent event) {
			switch (event.getEventType()) {
			case SerialPortEvent.DATA_AVAILABLE:
				serialPortDataReady();
				break;
			case SerialPortEvent.BI:
				break;
			case SerialPortEvent.CD:
				break;
			case SerialPortEvent.CTS:
				break;
			case SerialPortEvent.DSR:
				break;
			case SerialPortEvent.FE:
				break;
			case SerialPortEvent.OUTPUT_BUFFER_EMPTY:
				break;
			case SerialPortEvent.PE:
				break;
			case SerialPortEvent.RI:
				break;
			default:
			}
		}
	}
	
}
